#!/bin/sh
echo "=================================================================================================================================-================================="
echo "Welcome to the rooted Toon upgrade script. This script will try to upgrade your Toon using your original connection with Eneco. It will start the VPN if necessary."
echo "Please be advised that running this script is to your own risk!"
echo ""
echo "Version: 1.7 - ThehogNL - 8-3-2018"
echo ""
echo "==================================================================================================================================================================="
echo ""

#determine where this Toon is storing the update files
PKGCACHE='/HCBv2/tmp/opkg-cache'
if [ ! -d "$PKGCACHE" ] 
then
  PKGCACHE='/mnt/data/update'
fi

#determine current version
RUNNINGVERSION=`opkg list-installed base-qb2-\* | sed -r -e "s/base-qb2-([a-z]{3})\s-\s([0-9]*\.[0-9]*\.[0-9]*)-.*/\2/"`

VERSIONS=`/usr/bin/curl -Nks "https://notepad.pw/raw/6fmm2o8ev" | /usr/bin/tr '\n\r' ' ' | /bin/sed 's/.*#STARTTOONVERSIONS//' | /bin/sed 's/#ENDTOONVERSIONS.*//'`
if [ "$VERSIONS" == "" ]
then
 echo "Could not determine available versions from online sources. Using older well known verion list."
 #online versions list not available, falling back to a small well known list
 VERSIONS="2.9.26 3.0.29 3.0.32 3.1.22 3.2.14 3.2.18 3.3.8 3.4.4 3.5.4 3.6.3 3.7.8 3.7.9 4.3.20 4.4.21 4.7.23 4.8.25 4.9.23 4.10.6 4.11.6 4.12.0 4.13.6 4.13.7"
fi

if  echo $VERSIONS| grep -q $RUNNINGVERSION
then
  echo "You are currently running version "$RUNNINGVERSION
else
  echo "Unable to determine your current running version!"
  echo "DEBUG information:"
  echo "Detected: $RUNNINGVERSION"
  echo "Available: $VERSIONS"
  /usr/bin/opkg list-installed base-qb2-\*
  echo "END DEBUG information"
  exit
fi

echo ""
VERSION=""
echo "Which version do you want to upgrade to? You can look at the list of currently known Toon versions at http://pasted.co/d2a9b1dc"
read VERSION
while [ "$VERSION" == "" ]  || ! ( echo $VERSION | grep -qe '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*' )  || ! (echo $VERSIONS| grep -q $VERSION)
do
  echo "Please enter a valid version!"
  read VERSION
done


#determine current and next version levels and if it is allowed to upgrade to it

CURVERS_MAJOR="`echo $RUNNINGVERSION | sed -n -r -e 's,([0-9]+).([0-9]+).([0-9]+),\1,p'`"
CURVERS_MINOR="`echo $RUNNINGVERSION | sed -n -r -e 's,([0-9]+).([0-9]+).([0-9]+),\2,p'`"
CURVERS_BUILD="`echo $RUNNINGVERSION | sed -n -r -e 's,([0-9]+).([0-9]+).([0-9]+),\3,p'`"
VERS_MAJOR="`echo $VERSION | sed -n -r -e 's,([0-9]+).([0-9]+).([0-9]+),\1,p'`"
VERS_MINOR="`echo $VERSION | sed -n -r -e 's,([0-9]+).([0-9]+).([0-9]+),\2,p'`"
VERS_BUILD="`echo $VERSION | sed -n -r -e 's,([0-9]+).([0-9]+).([0-9]+),\3,p'`"

if [ $VERS_MAJOR -gt $CURVERS_MAJOR ] || [ $VERS_MAJOR -eq $CURVERS_MAJOR -a $VERS_MINOR -gt $CURVERS_MINOR ] || [ $VERS_MAJOR -eq $CURVERS_MAJOR -a $VERS_MINOR -eq $CURVERS_MINOR -a $VERS_BUILD -gt $CURVERS_BUILD ]
then
  if [ $CURVERS_MAJOR -ge 3 ] || [ $VERS_MAJOR -ge 3 -a $CURVERS_MAJOR -lt 3 -a "$RUNNINGVERSION" == "2.9.26" ] || [ $VERS_MAJOR -lt 3 ]
  then
    if  [ $VERS_MAJOR -le 4 -a $VERS_MINOR -le 10 ] || [ $VERS_MAJOR -ge 4 -a $VERS_MINOR -ge 11 -a "$RUNNINGVERSION" == "4.10.6" ] ||  [ $CURVERS_MAJOR -ge 4 -a  $CURVERS_MINOR -ge 11  ]
    then
      echo "Alright, I will try to upgrade to" $VERSION
    else
      echo "You need to upgrade to 4.10.16 first!"
      exit
    fi
  else
   echo "You need to upgrade to 2.9.26 first!"
   exit
  fi
else
  echo "Smartass.. "$VERSION" is not an upgrade for "$RUNNINGVERSION"!"
  exit
fi

#from version 4.11 and later the flavour is UNI (Universal). Tenants differences are now installed using packages.
FLAV="ene"
if [ $VERS_MAJOR -gt 4 ] || [ $VERS_MAJOR -eq 4 -a $VERS_MINOR -ge 11 ]
then 
  FLAV="uni"
fi

#save current iptables config 
/usr/sbin/iptables-save > /tmp/iptables.save

#and backup the default iptables file and passwd file
if [ ! -f /etc/default/iptables.conf ] 
then 
  echo "Your default iptables.conf (firewall configuration) is missing. I will restore it from the current running firewall config."
  /usr/sbin/iptables-save > /etc/default/iptables.conf
fi
/bin/cp /etc/default/iptables.conf /tmp/iptables.backup
if [ ! -f /etc/passwd ] 
then
  echo "Your password file (/etc/passwd) is missing. Please fix this before running this script."
  exit
fi
/bin/cp /etc/passwd /tmp/passwd.backup 

sync


#create a new iptables chain for this upgrade process and insert it in front of all rules
/usr/sbin/iptables -N UPDATE-INPUT
/usr/sbin/iptables -I INPUT -j UPDATE-INPUT

#allow icmp (ping) always, or else openvpn will restart all the time do to internal toon ping checks
/usr/sbin/iptables -A UPDATE-INPUT -p icmp -j ACCEPT
#drop all VPN traffic (for now)
/usr/sbin/iptables -A UPDATE-INPUT -i tap+ -j DROP
/usr/sbin/iptables -A UPDATE-INPUT -i tun+ -j DROP

#check if feed host is configured and there is a active route toward the host
#if openvpn is already running we don't need to start it manually, the FEEDHOST and FEEDROUTE should match then
FEEDHOST=`/bin/cat /etc/hosts | /bin/grep ^172 | /bin/grep feed | /usr/bin/awk 'BEGIN {FS="\t"}; {print $1}'| /usr/bin/awk 'BEGIN {FS="."}; {print $1"."$2"."$3}' `
FEEDROUTE=`/bin/ip route | /bin/grep ^172.*via.*tap0 | /usr/bin/awk '{print $1}'| /usr/bin/awk 'BEGIN {FS="."}; {print $1"."$2"."$3}'`
COUNT=0
while [ ! "$FEEDHOST" == "$FEEDROUTE" ] || [ "$FEEDHOST" = "" ] || [ "$FEEDROUTE" == "" ] ; do
  if [ $COUNT -gt 5 ] 
  then
    echo "Could not enable VPN in a normal reasonable time.. exiting!"
    echo "DEBUG information:"
    /bin/ip route
    /bin/cat /etc/hosts
    echo "END DEBUG information"
    /usr/bin/killall -9 openvpn
    /usr/sbin/iptables-restore <  /tmp/iptables.save
    exit
  fi
  COUNT=$((COUNT+1))
  /bin/echo "Waiting for tunnel to be alive and configured..."
  /usr/sbin/openvpn --config /etc/openvpn/vpn.conf --verb 0 >/dev/null --daemon 
  /bin/sleep 5
  FEEDHOST=`/bin/cat /etc/hosts | /bin/grep ^172 | /bin/grep feed | /usr/bin/awk 'BEGIN {FS="\t"}; {print $1}'| /usr/bin/awk 'BEGIN {FS="."}; {print $1"."$2"."$3}' `
  FEEDROUTE=`/bin/ip route | /bin/grep ^172.*via.*tap0 | /usr/bin/awk '{print $1}'| /usr/bin/awk 'BEGIN {FS="."}; {print $1"."$2"."$3}'`
done
/bin/echo "Tunnel is alive and configured. Now trying to download upgrade script..."

#set the feedhost correctly
FEEDHOST=`/bin/cat /etc/hosts | /bin/grep ^172 | /bin/grep feed | /usr/bin/awk 'BEGIN {FS="\t"}; {print $1}'`

#allow traffic from the vpn only from the feed host, and only if it is from the www port
#this blocks other traffic, most important blocking the service center so other changes are not pushed
/usr/sbin/iptables -I UPDATE-INPUT -p tcp -s $FEEDHOST -m tcp --sport 80 -j ACCEPT

#try to get the upgrade file from the feed host
/usr/bin/wget  http://feed.hae.int/feeds/qb2/upgrade/upgrade-qb2.sh -O $PKGCACHE/upgrade-qb2.sh -T 5 -t 2 -o /dev/null
RESULT=$?

if [ ! $RESULT == 0 ] ; then
 echo "Could not download the upgrade script from the source. Exiting..." 
 /usr/bin/killall -9 openvpn
 /usr/sbin/iptables-restore <  /tmp/iptables.save
 exit $RESULT
fi

#there is a valid upgrade available
echo "Upgrade script downloaded. Ready for upgrade process."

#make sure the upgrade script doesn't reboot after finishing
/bin/sed -i '/shutdown/c\#removed shutdown' $PKGCACHE/upgrade-qb2.sh 

#fixing /etc/hosts again, and change the offficial feed host to feed.hae.orig
sed -i 's/feed.hae.int/feed.hae.orig/' /etc/hosts
echo '127.0.0.1  feed.hae.int  feed' >> /etc/hosts

#rename the feed BASEURL host to the host we changed it to according to /etc/hosts
/bin/sed -i 's/feed.hae.int/feed.hae.orig/' $PKGCACHE/upgrade-qb2.sh 


/usr/bin/killall -9 openvpn
/usr/sbin/iptables-restore <  /tmp/iptables.save
exit

echo "Starting the upgrade prepare option which downloads all necessary files. No upgrade is done yet."

/bin/sh $PKGCACHE/upgrade-qb2.sh qb2 $FLAV $VERSION prepare

echo "Done preparing."


echo "Are your sure you want to upgrade to" $VERSION"? (answer with 'yes')"
read QUESTION
if [ ! "$QUESTION" == "yes" ] 
then
  echo "Ok, quitting the upgrade. It was a nice try tho..."
  /usr/bin/killall -9 openvpn
  /usr/sbin/iptables-restore <  /tmp/iptables.save
  exit
fi

echo "Starting the upgrade now! Just wait a while..."


echo "You can follow the upgrade if you open another SSH to the Toon and run this command:"
echo "tail -f "$PKGCACHE"/upgrade-qb2.sh.log"
/bin/sh $PKGCACHE/upgrade-qb2.sh qb2 $FLAV $VERSION execute

echo "Upgrade done!"

echo "Restoring your iptables and passwd files so you can login again after rebooting."
/bin/cp /tmp/iptables.backup /etc/default/iptables.conf
/bin/cp /tmp/passwd.backup /etc/passwd 

#cleaning up
/usr/bin/killall -9 openvpn
/usr/sbin/iptables-restore <  /tmp/iptables.save

sync

echo "Everything done! You should reboot now! But before that take some time to check if your /etc/passwd file is still valid and if /etc/default/iptables.conf is not blocking SSH access."
echo "Remember to update busybox from the forum if you upgraded to 4.10 or higher."
